@@ -6,25 +6,26 @@
 # http://www.popbill.com
 # Author : Jeong Yohan (code@linkhub.co.kr)
 # Written : 2015-03-24
-# Updated : 2017-03-02
+# Updated : 2019-10-24
 # Thanks for your interest.
 
-from .base import PopbillBase,PopbillException
+from .base import PopbillBase, PopbillException
+
 
 class CashbillService(PopbillBase):
     """ 팝빌 현금영수증 API Service Implementation. """
 
-    def __init__(self,LinkID,SecretKey):
+    def __init__(self, LinkID, SecretKey):
         """생성자
             args
                 LinkID : 링크허브에서 발급받은 링크아이디(LinkID)
                 SecretKeye 링크허브에서 발급받은 비밀키(SecretKey)
         """
 
-        super(self.__class__,self).__init__(LinkID,SecretKey)
+        super(self.__class__, self).__init__(LinkID, SecretKey)
         self._addScope("140")
 
-    def getChargeInfo(self, CorpNum, UserID = None):
+    def getChargeInfo(self, CorpNum, UserID=None):
         """ 과금정보 확인
             args
                 CorpNum : 회원 사업자번호
@@ -47,6 +48,8 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
+        if ToGo == None or ToGo == '':
+            raise PopbillException(-99999999, "TOGO값이 입력되지 않았습니다.")
 
         result = self._httpget('/Cashbill?TG=' + ToGo, CorpNum, UserID)
 
@@ -76,8 +79,8 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         try:
             result = self._httpget('/Cashbill/' + MgtKey, CorpNum)
@@ -89,7 +92,7 @@ class CashbillService(PopbillBase):
                 return False
             raise PE
 
-    def registIssue(self, CorpNum, cashbill, Memo, UserID = None):
+    def registIssue(self, CorpNum, cashbill, Memo, UserID=None, EmailSubject=None):
         """ 현금영수증 즉시발행
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -101,20 +104,21 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
         if cashbill == None:
-            raise PopbillException(-99999999,"현금영수증 정보가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "현금영수증 정보가 입력되지 않았습니다.")
 
         postData = ""
-        req = {}
 
         if Memo != None or Memo != '':
             cashbill.memo = Memo
 
+        if EmailSubject != None or EmailSubject != '':
+            cashbill.emailSubject = EmailSubject
+
         postData = self._stringtify(cashbill)
 
         return self._httppost('/Cashbill', postData, CorpNum, UserID, "ISSUE")
 
-
-    def register(self, CorpNum, cashbill, UserID = None):
+    def register(self, CorpNum, cashbill, UserID=None):
         """ 현금영수증 등록
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -126,14 +130,91 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
         if cashbill == None:
-            raise PopbillException(-99999999,"현금영수증 정보가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "현금영수증 정보가 입력되지 않았습니다.")
 
         postData = self._stringtify(cashbill)
 
-        return self._httppost('/Cashbill',postData,CorpNum,UserID)
+        return self._httppost('/Cashbill', postData, CorpNum, UserID)
 
+    def revokeRegistIssue(self, CorpNum, mgtKey, orgConfirmNum, orgTradeDate, smssendYN=False, memo=None, UserID=None,
+                          isPartCancel=False, cancelType=None, supplyCost=None, tax=None, serviceFee=None,
+                          totalAmount=None):
+        """ 취소현금영수증 즉시발행
+            args
+                CorpNum : 팝빌회원 사업자번호
+                mgtKey : 현금영수증 문서관리번호
+                orgConfirmNum : 원본현금영수증 승인번호
+                orgTradeDate : 원본현금영수증 거래일자
+                smssendYN : 발행안내문자 전송여부
+                memo : 메모
+                UserID : 팝빌회원 아이디
+                isPartCancel : 부분취소여부
+                cancelType : 취소사유
+                supplyCost : [취소] 공급가액
+                tax : [취소] 세액
+                serviceFee : [취소] 봉사료
+                totalAmount : [취소] 합계금액
+            return
+                처리결과. consist of code and message
+            raise
+                PopbillException
+        """
 
-    def update(self, CorpNum, MgtKey, cashbill, UserID = None):
+        postData = self._stringtify({
+            "mgtKey": mgtKey,
+            "orgConfirmNum": orgConfirmNum,
+            "orgTradeDate": orgTradeDate,
+            "smssendYN": smssendYN,
+            "memo": memo,
+            "isPartCancel": isPartCancel,
+            "cancelType": cancelType,
+            "supplyCost": supplyCost,
+            "tax": tax,
+            "serviceFee": serviceFee,
+            "totalAmount": totalAmount,
+        })
+
+        return self._httppost('/Cashbill', postData, CorpNum, UserID, "REVOKEISSUE")
+
+    def revokeRegister(self, CorpNum, mgtKey, orgConfirmNum, orgTradeDate, smssendYN=False, UserID=None,
+                       isPartCancel=False, cancelType=None, supplyCost=None, tax=None, serviceFee=None,
+                       totalAmount=None):
+        """ 취소현금영수증 임시저장
+            args
+                CorpNum : 팝빌회원 사업자번호
+                mgtKey : 현금영수증 문서관리번호
+                orgConfirmNum : 원본현금영수증 승인번호
+                orgTradeDate : 원본현금영수증 거래일자
+                smssendYN : 발행안내문자 전송여부
+                UserID : 팝빌회원 아이디
+                isPartCancel : 부분취소여부
+                cancelType : 취소사유
+                supplyCost : [취소] 공급가액
+                tax : [취소] 세액
+                serviceFee : [취소] 봉사료
+                totalAmount : [취소] 합계금액
+            return
+                처리결과. consist of code and message
+            raise
+                PopbillException
+        """
+
+        postData = self._stringtify({
+            "mgtKey": mgtKey,
+            "orgConfirmNum": orgConfirmNum,
+            "orgTradeDate": orgTradeDate,
+            "smssendYN": smssendYN,
+            "isPartCancel": isPartCancel,
+            "cancelType": cancelType,
+            "supplyCost": supplyCost,
+            "tax": tax,
+            "serviceFee": serviceFee,
+            "totalAmount": totalAmount,
+        })
+
+        return self._httppost('/Cashbill', postData, CorpNum, UserID, "REVOKE")
+
+    def update(self, CorpNum, MgtKey, cashbill, UserID=None):
         """ 수정
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -145,17 +226,16 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
         if cashbill == None:
-            raise PopbillException(-99999999,"현금영수증 정보가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "현금영수증 정보가 입력되지 않았습니다.")
 
         postData = self._stringtify(cashbill)
 
-        return self._httppost('/Cashbill/'+ MgtKey, postData, CorpNum, UserID, "PATCH")
+        return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "PATCH")
 
-
-    def issue(self, CorpNum, MgtKey, Memo = None, UserID = None):
+    def issue(self, CorpNum, MgtKey, Memo=None, UserID=None):
         """ 발행
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -168,8 +248,8 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         postData = ""
         req = {}
@@ -181,7 +261,7 @@ class CashbillService(PopbillBase):
 
         return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "ISSUE")
 
-    def cancelIssue(self, CorpNum, MgtKey, Memo = None, UserID = None):
+    def cancelIssue(self, CorpNum, MgtKey, Memo=None, UserID=None):
         """ 발행취소
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -194,8 +274,8 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         postData = ""
         req = {}
@@ -207,7 +287,7 @@ class CashbillService(PopbillBase):
 
         return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "CANCELISSUE")
 
-    def delete(self, CorpNum, MgtKey, UserID = None):
+    def delete(self, CorpNum, MgtKey, UserID=None):
         """ 삭제
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -219,12 +299,13 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
-        return self._httppost('/Cashbill/' + MgtKey,'', CorpNum, UserID, "DELETE")
+        return self._httppost('/Cashbill/' + MgtKey, '', CorpNum, UserID, "DELETE")
 
-    def search(self,CorpNum,DType,SDate,EDate,State,TradeType,TradeUsage,TaxationType,Page,PerPage,Order,UserID=None,QString=None) :
+    def search(self, CorpNum, DType, SDate, EDate, State, TradeType, TradeUsage, TaxationType, Page, PerPage, Order,
+               UserID=None, QString=None, TradeOpt=None):
         """ 목록 조회
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -232,16 +313,26 @@ class CashbillService(PopbillBase):
                 SDate : 시작일자, 표시형식(yyyyMMdd)
                 EDate : 종료일자, 표시형식(yyyyMMdd)
                 State : 상태코드 배열, 2,3번째 자리에 와일드카드(*) 사용가능
-                TradeType : 현금영수증 형태 배열, N-일반현금영수증, C-취소현금영수증
-                TradeUsage : 거래용도 배열, P-소득공제용, C-지출증빙용
+                TradeType : 문서형태 배열, N-일반현금영수증, C-취소현금영수증
+                TradeUsage : 거래구분 배열, P-소득공제용, C-지출증빙용
                 TaxationType : 과세형태 배열, T-과세, N-비과세
                 Page : 페이지번호
-                PerPage : 페이지당 목록개수
+                PerPage : 페이지당 검색개수
                 Order : 정렬방향, D-내림차순, A-오름차순
                 UserID : 팝빌 회원아이디
                 QString : 현금영수증 식별번호, 미기재시 전체조회
+                TradeOpt : 거래유형, N-일반, B-도서공연, T-대중교통
         """
 
+        if DType == None or DType == '':
+            raise PopbillException(-99999999, "일자유형이 입력되지 않았습니다.")
+
+        if SDate == None or SDate == '':
+            raise PopbillException(-99999999, "시작일자가 입력되지 않았습니다.")
+
+        if EDate == None or EDate == '':
+            raise PopbillException(-99999999, "종료일자가 입력되지 않았습니다.")
+
         uri = '/Cashbill/Search'
         uri += '?DType=' + DType
         uri += '&SDate=' + SDate
@@ -254,10 +345,13 @@ class CashbillService(PopbillBase):
         uri += '&PerPage=' + str(PerPage)
         uri += '&Order=' + Order
 
-        if QString is not None :
+        if QString is not None:
             uri += '&QString=' + QString
 
-        return self._httpget(uri, CorpNum,UserID)
+        if TradeOpt is not None:
+            uri += '&TradeOpt=' + ','.join(TradeOpt)
+
+        return self._httpget(uri, CorpNum, UserID)
 
     def getInfo(self, CorpNum, MgtKey):
         """ 상태/요약 정보 조회
@@ -270,12 +364,12 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         return self._httpget('/Cashbill/' + MgtKey, CorpNum)
 
-    def getInfos(self,CorpNum,MgtKeyList):
+    def getInfos(self, CorpNum, MgtKeyList):
         """ 상태정보 다량 확인, 최대 1000건
             args
                 CorpNum : 회원 사업자 번호
@@ -286,11 +380,11 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
         if MgtKeyList == None or len(MgtKeyList) < 1:
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         postData = self._stringtify(MgtKeyList)
 
-        return self._httppost('/Cashbill/States',postData,CorpNum)
+        return self._httppost('/Cashbill/States', postData, CorpNum)
 
     def getDetailInfo(self, CorpNum, MgtKey):
         """ 상세정보 조회
@@ -303,12 +397,12 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         return self._httpget('/Cashbill/' + MgtKey + '?Detail', CorpNum)
 
-    def sendEmail(self, CorpNum, MgtKey, ReceiverEmail, UserID = None):
+    def sendEmail(self, CorpNum, MgtKey, ReceiverEmail, UserID=None):
         """ 알림메일 재전송
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -321,16 +415,16 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
-        if ReceiverEmail == None or ReceiverEmail == "" :
-            raise PopbillException(-99999999,"수신자 메일주소가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
+        if ReceiverEmail == None or ReceiverEmail == "":
+            raise PopbillException(-99999999, "수신자 메일주소가 입력되지 않았습니다.")
 
-        postData = self._stringtify({"receiver" : ReceiverEmail})
+        postData = self._stringtify({"receiver": ReceiverEmail})
 
         return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "EMAIL")
 
-    def sendSMS(self, CorpNum, MgtKey, Sender, Receiver, Contents, UserID = None):
+    def sendSMS(self, CorpNum, MgtKey, Sender, Receiver, Contents, UserID=None):
         """ 알림문자 전송
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -345,20 +439,20 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
 
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
-        if Receiver == None or Receiver == "" :
-            raise PopbillException(-99999999,"문자 수신번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
+        if Receiver == None or Receiver == "":
+            raise PopbillException(-99999999, "문자 수신번호가 입력되지 않았습니다.")
 
         postData = self._stringtify({
-                                    "sender" : Sender,
-                                    "receiver" : Receiver,
-                                    "contents" : Contents
-                                    })
+            "sender": Sender,
+            "receiver": Receiver,
+            "contents": Contents
+        })
 
         return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "SMS")
 
-    def sendFAX(self, CorpNum, MgtKey, Sender, Receiver, UserID = None):
+    def sendFAX(self, CorpNum, MgtKey, Sender, Receiver, UserID=None):
         """ 현금영수증 팩스전송
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -371,19 +465,18 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
-        if Receiver == None or Receiver == "" :
-            raise PopbillException(-99999999,"팩스 수신번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
+        if Receiver == None or Receiver == "":
+            raise PopbillException(-99999999, "팩스 수신번호가 입력되지 않았습니다.")
 
         postData = self._stringtify({
-                                        "sender" : Sender,
-                                        "receiver" : Receiver
-                                    })
+            "sender": Sender,
+            "receiver": Receiver
+        })
 
         return self._httppost('/Cashbill/' + MgtKey, postData, CorpNum, UserID, "FAX")
 
-
     def getLogs(self, CorpNum, MgtKey):
         """ 문서이력 조회
             args
@@ -394,12 +487,12 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         return self._httpget('/Cashbill/' + MgtKey + '/Logs', CorpNum)
 
-    def getPopUpURL(self, CorpNum, MgtKey, UserID = None):
+    def getPopUpURL(self, CorpNum, MgtKey, UserID=None):
         """ 현금영수증 1장의 팝빌 화면을 볼수있는 팝업 URL 확인
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -410,14 +503,14 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         result = self._httpget('/Cashbill/' + MgtKey + '?TG=POPUP', CorpNum, UserID)
 
         return result.url
 
-    def getPrintURL(self, CorpNum, MgtKey, UserID = None):
+    def getPrintURL(self, CorpNum, MgtKey, UserID=None):
         """ 공급자용 인쇄 URL 확인
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -428,15 +521,14 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         result = self._httpget('/Cashbill/' + MgtKey + '?TG=PRINT', CorpNum, UserID)
 
         return result.url
 
-
-    def getEPrintURL(self, CorpNum, MgtKey, UserID = None):
+    def getEPrintURL(self, CorpNum, MgtKey, UserID=None):
         """ 공급받는자용 인쇄 URL 확인
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -447,15 +539,14 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         result = self._httpget('/Cashbill/' + MgtKey + '?TG=EPRINT', CorpNum, UserID)
 
         return result.url
 
-
-    def getMailURL(self, CorpNum, MgtKey, UserID = None):
+    def getMailURL(self, CorpNum, MgtKey, UserID=None):
         """ 공급받는자용 메일 링크 URL 확인
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -466,15 +557,14 @@ class CashbillService(PopbillBase):
             raise
                 PopbillException
         """
-        if MgtKey == None or MgtKey == "" :
-            raise PopbillException(-99999999,"관리번호가 입력되지 않았습니다.")
+        if MgtKey == None or MgtKey == "":
+            raise PopbillException(-99999999, "관리번호가 입력되지 않았습니다.")
 
         result = self._httpget('/Cashbill/' + MgtKey + '?TG=MAIL', CorpNum, UserID)
 
         return result.url
 
-
-    def getMassPrintURL(self, CorpNum, MgtKeyList, UserID = None):
+    def getMassPrintURL(self, CorpNum, MgtKeyList, UserID=None):
         """ 다량 인쇄 URL 확인
             args
                 CorpNum : 팝빌회원 사업자번호
@@ -486,7 +576,7 @@ class CashbillService(PopbillBase):
                 PopbillException
         """
         if MgtKeyList == None:
-            raise PopbillException(-99999999,"관리번호 배열이 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "관리번호 배열이 입력되지 않았습니다.")
 
         postData = self._stringtify(MgtKeyList)
 
@@ -494,6 +584,40 @@ class CashbillService(PopbillBase):
 
         return result.url
 
+    def listEmailConfig(self, CorpNum, UserID=None):
+        """ 알림메일 전송목록 조회
+            args
+                CorpNum : 팝빌회원 사업자번호
+                UserID : 팝빌회원 아이디
+            return
+               현금영수증 관련 메일전송 항목에 대한 전송여부 목록
+            raise
+                PopbillException
+        """
+        return self._httpget('/Cashbill/EmailSendConfig', CorpNum, UserID)
+
+    def updateEmailConfig(self, Corpnum, EmailType, SendYN, UserID=None):
+        """ 알림메일 전송설정 수정
+            args
+                CorpNum : 팝빌회원 사업자번호
+                EmailType: 메일전송유형
+                SendYN: 전송여부 (True-전송, False-미전송)
+                UserID : 팝빌회원 아이디
+            return
+               처리결과. consist of code and message
+            raise
+                PopbillException
+        """
+        if EmailType == None or EmailType == '':
+            raise PopbillException(-99999999, "메일전송 타입이 입력되지 않았습니다.")
+
+        if SendYN == None or SendYN == '':
+            raise PopbillException(-99999999, "메일전송 여부 항목이 입력되지 않았습니다.")
+
+        uri = "/Cashbill/EmailSendConfig?EmailType=" + EmailType + "&SendYN=" + str(SendYN)
+        return self._httppost(uri, "", Corpnum, UserID)
+
+
 class Cashbill(object):
-    def __init__(self,**kwargs):
+    def __init__(self, **kwargs):
         self.__dict__ = kwargs
